/*
 * Copyright (C) 2016 Väinö Helminen
 */

#include <stddef.h>
#include "favicon.h"
#include "log.h"

#ifdef __WIN32__
#include <stddef.h>
#include <windows.h>

const char *FAVICON = NULL;
unsigned FAVICON_SIZE = 0;

#pragma pack(push)
#pragma pack(2)
typedef struct _GRPICONDIRENTRY {
  BYTE bWidth;
  BYTE bHeight;
  BYTE bColorCount;
  BYTE bReserved;
  WORD wPlanes;
  WORD wBitCount;
  DWORD dwBytesInRes;
  WORD nID;
} GRPICONDIRENTRY;

typedef struct _GRPICONDIR {
  WORD idReserved;
  WORD idType;
  WORD idCount;
  GRPICONDIRENTRY idEntries[1];
} GRPICONDIR;

typedef struct _ICONDIRENTRY {
  BYTE bWidth;
  BYTE bHeight;
  BYTE bColorCount;
  BYTE bReserved;
  WORD wPlanes;
  WORD wBitCount;
  DWORD dwBytesInRes;
  DWORD dwImageOffset;
} ICONDIRENTRY;

typedef struct _ICONDIR {
  WORD idReserved;
  WORD idType;
  WORD idCount;
  ICONDIRENTRY idEntries[1];
} ICONDIR;
#pragma pack(pop)

void load_favicon_ico()
{
  trace("FindResource");
  HRSRC hRIconGroup = FindResource(NULL, MAKEINTRESOURCE(0), RT_GROUP_ICON);
  if(!hRIconGroup) {
    dllerror("FindResource");
    exit(EXIT_FAILURE);
  }

  trace("LoadResource");
  HGLOBAL hIconGroup = LoadResource(NULL, hRIconGroup);
  if(!hIconGroup) {
    dllerror("LoadResource");
    exit(EXIT_FAILURE);
  }

  trace("LockResource");
  GRPICONDIR *iconGroupDir = (void*)LockResource(hIconGroup);
  if(!iconGroupDir) {
    dllerror("LockResource");
    exit(EXIT_FAILURE);
  }

  const int count = iconGroupDir->idCount;
  size_t size = sizeof(ICONDIR) + (count-1) * sizeof(ICONDIRENTRY), offset = size;
  for(int i = 0; i < count; ++i) {
    size += iconGroupDir->idEntries[i].dwBytesInRes;
  }

  trace("malloc");
  char *data = malloc(size);
  if(!data) {
    perror("malloc");
    exit(EXIT_FAILURE);
  }

  ICONDIR *iconDir = (void*)data;
  iconDir->idReserved = 0;
  iconDir->idType = 1; // ICON
  iconDir->idCount = count;

  for(int i = 0; i < count; ++i) {
    const size_t iconSize = iconGroupDir->idEntries[i].dwBytesInRes;
    iconDir->idEntries[i].bWidth = iconGroupDir->idEntries[i].bWidth;
    iconDir->idEntries[i].bHeight = iconGroupDir->idEntries[i].bHeight;
    iconDir->idEntries[i].bColorCount = iconGroupDir->idEntries[i].bColorCount;
    iconDir->idEntries[i].bReserved = 0;
    iconDir->idEntries[i].wPlanes = iconGroupDir->idEntries[i].wPlanes;
    iconDir->idEntries[i].wBitCount = iconGroupDir->idEntries[i].wBitCount;
    iconDir->idEntries[i].dwBytesInRes = iconSize;
    iconDir->idEntries[i].dwImageOffset = offset;

    trace("FindResource");
    HRSRC hRIcon = FindResource(NULL, MAKEINTRESOURCE(iconGroupDir->idEntries[i].nID), RT_ICON);
    if(!hRIcon) {
      dllerror("FindResource");
      exit(EXIT_FAILURE);
    }

    trace("LoadResource");
    HGLOBAL hIcon = LoadResource(NULL, hRIcon);
    if(!hIcon) {
      dllerror("LoadResource");
      exit(EXIT_FAILURE);
    }

    trace("LockResource");
    const char *iconData = LockResource(hIcon);
    if(!iconData) {
      dllerror("LockResource");
      exit(EXIT_FAILURE);
    }

    trace("memcpy");
    memcpy(data + offset, iconData, iconSize);
    offset += iconSize;

    /* obsolete, so not bothering to check return value which is
       always false */
    trace("FreeResource");
    FreeResource(hIcon);
  }

  assert(size == offset);

  /* obsolete, so not bothering to check return value which is always
     false */
  trace("FreeResource");
  FreeResource(hIconGroup);

  FAVICON = data;
  FAVICON_SIZE = size;
}

#else
#  include "../favicon.ico.h"
const unsigned FAVICON_SIZE = sizeof FAVICON;

#endif
