/*
 * Copyright (C) 2016 Väinö Helminen
 */

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include "list.h"
#include "log.h"

static List* list_alloc()
{
  List *list = NULL;
  list = malloc(sizeof *list);
  if (!list) {
    perror("malloc");
    exit(EXIT_FAILURE);
  }
  list->value = NULL;
  list->next = NULL;
  return list;
}

void list_free(List **list, const bool free_values)
{
  assert(list);
  List *node = *list;
  *list = NULL;
  while(node) {
    List *next = node->next;
    node->next = NULL;
    if(free_values) {
      free(node->value);
    }
    node->value = NULL;
    free(node);
    node = next;
  }
}

size_t list_length(const List *list)
{
  assert(list);

  size_t length = 0;
  while(list) {
    list = list->next;
    ++length;
  }
  return length;
}

List* list_tail(List *list)
{
  while(list && list->next) {
    list = list->next;
  }
  return list;
}

void list_push(List **list, void* value)
{
  assert(list);

  List *node = list_alloc();
  node->value = value;
  node->next = *list;
  *list = node;
}

void* list_pop(List** list)
{
  assert(list);

  List *head = *list;
  if(!head) return NULL;

  *list = head->next;
  void *value = head->value;
  head->next = NULL;
  free(head);
  return value;
}

bool list_remove(List **list, void *value)
{
  assert(list);

  List *node = *list;
  while(node) {
    if(node->value == value) break;
    list = &node->next;
    node = node->next;
  }

  debug("success: %s", node ? "true" : "false");
  if(!node) return false;

  *list = node->next;
  node->value = NULL;
  node->next = NULL;

  trace("free");
  free(node);
  return true;
}

void list_reverse(List **list)
{
  assert(list);

  List *node = *list, *prev = NULL, *next = NULL;
  while(node) {
    next = node->next;
    node->next = prev;
    prev = node;
    node = next;
  }
  *list = prev;
}
