/*
 * Copyright (C) 2016 Väinö Helminen
 */

#include <stdio.h>
#include <string.h>
#include <errno.h>
#include "log.h"

int LOG_LEVEL = DEFAULT_LOG_LEVEL;

void _logger(int level, const char *file, const int line, const char *func, const char *format, ...)
{
  if(level < LOG_ASSERT || level > LOG_TRACE) {
    level = LOG_ERROR;
  }
  if(level > LOG_LEVEL) {
    return;
  }
  fprintf(stderr, "%c [%s:%d:%s] ", "AEWIDT"[level], file, line, func);
  va_list ap;
  va_start(ap, format);
  vfprintf(stderr, format, ap);
  va_end(ap);
  fputs("\n", stderr);
}

void _logger_strerror(const char *file, const int line, const char *func, const char *s, const int error)
{
  _logger(1, file, line, func, "%s: %d %s", s, error, strerror(error));
}

#ifdef __WIN32__

void _win32_error(const char *file, const int line, const char *func, const char *s, const DWORD error)
{
  char *tmp = NULL;
  FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_IGNORE_INSERTS,
                NULL, error, MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
                (LPTSTR)&tmp, 0, NULL);
  _logger(1, file, line, func, "%s: %d %s", s, (int)error, tmp);
  LocalFree(tmp);
}

void _socketerror(const char *file, const int line, const char *func, const char *s)
{
  _win32_error(file, line, func, s, WSAGetLastError());
}

void _dllerror(const char *file, const int line, const char *func, const char *s)
{
  _win32_error(file, line, func, s, GetLastError());
}

#else
#include <time.h>

double monotime()
{
#ifndef CLOCK_MONOTONIC_RAW
#define CLOCK_MONOTONIC_RAW CLOCK_MONOTONIC
#endif
  struct timespec ts;
  trace("clock_gettime");
  if(clock_gettime(CLOCK_MONOTONIC_RAW, &ts)) {
    perror("clock_gettime");
    return 0;
  }
  return ts.tv_sec + ts.tv_nsec * 1e-9;
}

#endif
