/*
 * Copyright (C) 2016 Väinö Helminen
 */
#ifndef LOG_H_HM5SV7L9
#define LOG_H_HM5SV7L9

#include <stdarg.h>
#include <errno.h>

#define LOG_ASSERT 0
#define LOG_ERROR  1
#define LOG_WARN   2
#define LOG_INFO   3
#define LOG_DEBUG  4
#define LOG_TRACE  5

#undef perror
#undef assert

extern int LOG_LEVEL;

void _logger(int level, const char *file, const int line, const char *func, const char *format, ...);
void _logger_strerror(const char *file, const int line, const char *func, const char *s, const int error);

#define error(...) _logger(LOG_ERROR, __FILE__, __LINE__, __func__, __VA_ARGS__)

#define perror(s) _logger_strerror(__FILE__, __LINE__, __func__, s, errno)

#if defined(DEBUG) && !defined(NDEBUG)
#  define DEFAULT_LOG_LEVEL LOG_DEBUG
#  define assert(cond) if(!(cond)) _logger(LOG_ASSERT, __FILE__, __LINE__, __func__, #cond);
#  define warn(...) _logger(LOG_WARN, __FILE__, __LINE__, __func__, __VA_ARGS__)
#  define info(...) _logger(LOG_INFO, __FILE__, __LINE__, __func__, __VA_ARGS__)
#  define debug(...) _logger(LOG_DEBUG, __FILE__, __LINE__, __func__, __VA_ARGS__)
#  define trace(...) _logger(LOG_TRACE, __FILE__, __LINE__, __func__, __VA_ARGS__)
#else
#  define DEFAULT_LOG_LEVEL LOG_ERROR
#  define assert(...) do{ }while(0)
#  define warn(...) do{ }while(0)
#  define info(...) do{ }while(0)
#  define debug(...) do {}while(0)
#  define trace(...) do {}while(0)
#endif


#ifdef __WIN32__
#  include <windows.h>
#  define win32error(s, error) _win32_error(__FILE__, __LINE__, __func__, s, error)
#  define socketerror(s) _socketerror(__FILE__, __LINE__, __func__, s)
#  define dllerror(s) _dllerror(__FILE__, __LINE__, __func__, s)

void _win32_error(const char *file, const int line, const char *func, const char *s, const DWORD error);
void _socketerror(const char *file, const int line, const char *func, const char *s);
void _dllerror(const char *file, const int line, const char *func, const char *s);

#else

double monotime(void);

#endif


#endif
