/*
 * Copyright (C) 2016 Väinö Helminen
 */

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include "strbuf.h"

StringBuffer* strbuf_alloc(size_t size)
{
  StringBuffer *sb = malloc(sizeof *sb);
  if(!sb) {
    perror("malloc");
    exit(EXIT_FAILURE);
  }
  sb->size = 0;
  sb->buffer_size = 0;
  sb->buffer = NULL;
  strbuf_realloc(sb, size ? size : 256);
  return sb;
}

void strbuf_realloc(StringBuffer *sb, size_t size)
{
  size_t used = sb->size;
  if(used > size) {
    used = size;
  }

  char *buffer = malloc(size + 1);
  if(!buffer) {
    perror("malloc");
    exit(EXIT_FAILURE);
  }

  if(sb->buffer) {
    memcpy(buffer, sb->buffer, used);
    free(sb->buffer);
  }
  buffer[used] = '\0';

  sb->size = used;
  sb->buffer_size = size;
  sb->buffer = buffer;
}

void strbuf_free(StringBuffer **sb)
{
  (*sb)->size = 0;
  (*sb)->buffer_size = 0;
  free((*sb)->buffer);
  (*sb)->buffer = NULL;
  free(*sb);
  *sb = NULL;
}

void strbuf_truncate(StringBuffer *sb, size_t size)
{
  if(sb->size > size) {
    sb->size = size;
    sb->buffer[size] = '\0';
  }
}

void strbuf_copy(StringBuffer *sb, const char *buf, const size_t size)
{
  if(sb->size + size > sb->buffer_size) {
    strbuf_realloc(sb, sb->buffer_size * 2);
  }
  memcpy(sb->buffer + sb->size, buf, size);
  sb->size += size;
  sb->buffer[sb->size] = '\0';
}

void strbuf_append(StringBuffer *sb, const char *str)
{
  strbuf_copy(sb, str, strlen(str));
}

void strbuf_addchar(StringBuffer *sb, const int c)
{
  if(sb->size + 1> sb->buffer_size) {
    strbuf_realloc(sb, sb->buffer_size * 2);
  }
  sb->buffer[sb->size++] = c;
  sb->buffer[sb->size] = '\0';
}


void strbuf_format(StringBuffer *sb, const char *format, ...)
{
  va_list ap, ap2;
  va_start(ap, format);
  va_copy(ap2, ap);
  int size = vsnprintf(NULL, 0, format, ap2);
  if(size < 0) {
    perror("vsnprintf");
    exit(EXIT_FAILURE);
  }
  va_end(ap2);

  if(sb->size + size > sb->buffer_size) {
    strbuf_realloc(sb, sb->buffer_size * 2);
  }

  size = vsnprintf(sb->buffer + sb->size, size + 1, format, ap);
  if(size < 0) {
    perror("vsnprintf");
    exit(EXIT_FAILURE);
  }
  va_end(ap);

  sb->size += size;
}
