/*
 * Copyright (C) 2016 Väinö Helminen
 */
#include <stdlib.h>
#include <errno.h>
#ifdef __WIN32__
#  include <windows.h>
#  include <process.h>
#else
#  include <pthread.h>
#endif
#include "thread.h"
#include "log.h"

int thread_create(void (*run)(void*), void *arg)
{
  int ret = 0;
#ifdef __WIN32__
  trace("_beginthread");
  HANDLE tid = (HANDLE)_beginthread(run, 0, arg);
  if(tid == (HANDLE)-1) {
    ret = errno;
    _logger_strerror(__FILE__, __LINE__, __func__, "_beginthread", ret);
  }
#else
  pthread_t tid;
  pthread_attr_t attr;
  pthread_attr_init(&attr);
  pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED);

  trace("pthread_create");;
  ret = pthread_create(&tid, &attr, (void*)run, arg);
  if(ret) {
    _logger_strerror(__FILE__, __LINE__, __func__, "pthread_create", ret);
  }

  pthread_attr_destroy(&attr);
#endif
  return ret;
}
